[llvm-commits] [llvm] r52731 - in /llvm/trunk: include/llvm/System/Memory.h lib/System/Memory.cpp

Chris Lattner sabre at nondot.org
Wed Jun 25 10:14:12 PDT 2008


Author: lattner
Date: Wed Jun 25 12:14:10 2008
New Revision: 52731

URL: http://llvm.org/viewvc/llvm-project?rev=52731&view=rev
Log:
Add a new InvalidateInstructionCache method to sys::Memory.

Modified:
    llvm/trunk/include/llvm/System/Memory.h
    llvm/trunk/lib/System/Memory.cpp

Modified: llvm/trunk/include/llvm/System/Memory.h
URL: http://llvm.org/viewvc/llvm-project/llvm/trunk/include/llvm/System/Memory.h?rev=52731&r1=52730&r2=52731&view=diff

==============================================================================
--- llvm/trunk/lib/System/Memory.cpp (original)
+++ llvm/trunk/lib/System/Memory.cpp Wed Jun 25 12:14:10 2008
@@ -17,12 +17,6 @@
 
 namespace llvm {
 using namespace sys;
-
-//===----------------------------------------------------------------------===//
-//=== WARNING: Implementation here must contain only TRULY operating system
-//===          independent code.
-//===----------------------------------------------------------------------===//
-
 }
 
 // Include the platform-specific parts of this class.
@@ -32,3 +26,34 @@
 #ifdef LLVM_ON_WIN32
 #include "Win32/Memory.inc"
 #endif
+
+/// InvalidateInstructionCache - Before the JIT can run a block of code
+/// that has been emitted it must invalidate the instruction cache on some
+/// platforms.
+void llvm::sys::Memory::InvalidateInstructionCache(const void *Addr,
+                                                   size_t Len) {
+  
+// icache invalidation for PPC.
+#if (defined(__POWERPC__) || defined (__ppc__) || \
+     defined(_POWER) || defined(_ARCH_PPC))
+   #if defined(__APPLE__)
+       extern "C" void sys_icache_invalidate(const void *Addr, size_t len);
+       sys_icache_invalidate(Addr, len);
+   #elif defined(__GNUC__)
+        const size_t LineSize = 32;
+
+        const intptr_t Mask = ~(LineSize - 1);
+        const intptr_t StartLine = ((intptr_t) Addr) & Mask;
+        const intptr_t EndLine = ((intptr_t) Addr + len + LineSize - 1) & Mask;
+
+        for (intptr_t Line = StartLine; Line < EndLine; Line += LineSize)
+          asm volatile("dcbf 0, %0" : : "r"(Line));
+        asm volatile("sync");
+
+        for (intptr_t Line = StartLine; Line < EndLine; Line += LineSize)
+          asm volatile("icbi 0, %0" : : "r"(Line));
+        asm volatile("isync");
+   #endif
+#endif  // end PPC
+
+}
\ No newline at end of file





More information about the llvm-commits mailing list
--- llvm/trunk/include/llvm/System/Memory.h (original)
+++ llvm/trunk/include/llvm/System/Memory.h Wed Jun 25 12:14:10 2008
@@ -39,33 +39,36 @@
   /// @since 1.4
   /// @brief An abstraction for memory operations.
   class Memory {
-    /// @name Functions
-    /// @{
-    public:
-      /// This method allocates a block of Read/Write/Execute memory that is
-      /// suitable for executing dynamically generated code (e.g. JIT). An
-      /// attempt to allocate \p NumBytes bytes of virtual memory is made.
-      /// \p NearBlock may point to an existing allocation in which case
-      /// an attempt is made to allocate more memory near the existing block.
-      ///
-      /// On success, this returns a non-null memory block, otherwise it returns
-      /// a null memory block and fills in *ErrMsg.
-      /// 
-      /// @brief Allocate Read/Write/Execute memory.
-      static MemoryBlock AllocateRWX(unsigned NumBytes,
-                                     const MemoryBlock *NearBlock,
-                                     std::string *ErrMsg = 0);
+  public:
+    /// This method allocates a block of Read/Write/Execute memory that is
+    /// suitable for executing dynamically generated code (e.g. JIT). An
+    /// attempt to allocate \p NumBytes bytes of virtual memory is made.
+    /// \p NearBlock may point to an existing allocation in which case
+    /// an attempt is made to allocate more memory near the existing block.
+    ///
+    /// On success, this returns a non-null memory block, otherwise it returns
+    /// a null memory block and fills in *ErrMsg.
+    /// 
+    /// @brief Allocate Read/Write/Execute memory.
+    static MemoryBlock AllocateRWX(unsigned NumBytes,
+                                   const MemoryBlock *NearBlock,
+                                   std::string *ErrMsg = 0);
 
-      /// This method releases a block of Read/Write/Execute memory that was
-      /// allocated with the AllocateRWX method. It should not be used to
-      /// release any memory block allocated any other way.
-      ///
-      /// On success, this returns false, otherwise it returns true and fills
-      /// in *ErrMsg.
-      /// @throws std::string if an error occurred.
-      /// @brief Release Read/Write/Execute memory.
-      static bool ReleaseRWX(MemoryBlock &block, std::string *ErrMsg = 0);
-    /// @}
+    /// This method releases a block of Read/Write/Execute memory that was
+    /// allocated with the AllocateRWX method. It should not be used to
+    /// release any memory block allocated any other way.
+    ///
+    /// On success, this returns false, otherwise it returns true and fills
+    /// in *ErrMsg.
+    /// @throws std::string if an error occurred.
+    /// @brief Release Read/Write/Execute memory.
+    static bool ReleaseRWX(MemoryBlock &block, std::string *ErrMsg = 0);
+    
+    
+    /// InvalidateInstructionCache - Before the JIT can run a block of code
+    /// that has been emitted it must invalidate the instruction cache on some
+    /// platforms.
+    static void InvalidateInstructionCache(const void *Addr, size_t Len);
   };
 }
 }

Modified: llvm/trunk/lib/System/Memory.cpp
URL: http://llvm.org/viewvc/llvm-project/llvm/trunk/lib/System/Memory.cpp?rev=52731&r1=52730&r2=52731&view=diff

==============================================================================